open Bigarray

let assert_sorted cmp (a : BigQuicksort.array_type) left right =
  for i=left to right-2 do
    assert (cmp (Array1.get a i) (Array1.get a (i+1)) <= 0)
  done

let checksum (a : BigQuicksort.array_type) left right : int =
  let sum = ref 0 in
  for i=left to right-1 do
    sum := !sum lxor (Array1.get a i)
  done;
  !sum

let random_array seed max n : BigQuicksort.array_type =
  let prng = Random.State.make seed in
  Array1.init Int C_layout n (fun _ -> Random.State.int prng max);; 

let print_array oc prt a left right =
  for i=left to right-1 do
    Printf.fprintf oc "[%d] = %a\n" i prt (Array1.get a i)
  done

let output_int oc n =
  output_string oc (string_of_int n);;

module Tsk = Domainslib__Task;;

let cores = 4 in
let pool = Tsk.setup_pool ~num_additional_domains:(cores-1) () in
let run f = Tsk.run pool f in

(*
let run f = f () in
 *)

let n1 = ref 1 in
while !n1 <= 10_000_001 do
  let n = !n1 in
  n1 := n + 1000;
  Gc.full_major ();
  
  let max = 1_000_000_000
  and seed = [|n; 2; 3|] in
  let a = random_array seed max n in
  let sum = checksum a 0 n in
  let times,() = Timings.time_computation (fun () ->
                     run
                     (fun () -> BigQuicksort.quicksort ( - ) a 0 n)) in
  assert_sorted ( - ) a 0 n ;
  let sum2 = checksum a 0 n in
  assert (sum = sum2);
  Printf.printf "%d\t%f\t%f\n" n times.cpu_time times.wall_time;
  flush stdout
done;;
